@@ -1,3 +1,19 @@
+//@HEADER
+// ************************************************************************
+//
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
+//
+// Under the terms of Contract DE-NA0003525 with NTESS,
+// the U.S. Government retains certain rights in this software.
+//
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+//
+//@HEADER
+
 #include <Kokkos_Core.hpp>
 #include <gtest/gtest.h>
 #include <PerfTest_Category.hpp>
@@ -26,8 +42,7 @@ bool is_overlapping<Kokkos::Cuda>(const Kokkos::Cuda&) {
 
 #ifdef KOKKOS_ENABLE_HIP
 template <>
-bool is_overlapping<Kokkos::Experimental::HIP>(
-    const Kokkos::Experimental::HIP&) {
+bool is_overlapping<Kokkos::HIP>(const Kokkos::HIP&) {
   // FIXME_HIP This doesn't pass yet in CI.
   return false;
   // bool value          = true;
