@@ -1,46 +1,18 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 3.0
-//       Copyright (2020) National Technology & Engineering
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
 //               Solutions of Sandia, LLC (NTESS).
 //
 // Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-//
-// ************************************************************************
 //@HEADER
-*/
 
 /*--------------------------------------------------------------------------*/
 /* Kokkos interfaces */
@@ -202,7 +174,9 @@ void cuda_internal_error_abort(cudaError e, const char *name, const char *file,
   if (file) {
     out << " " << file << ":" << line;
   }
-  abort(out.str().c_str());
+  // FIXME Call Kokkos::Impl::host_abort instead of Kokkos::abort to avoid a
+  // warning about Kokkos::abort returning in some cases.
+  host_abort(out.str().c_str());
 }
 
 //----------------------------------------------------------------------------
@@ -291,12 +265,14 @@ const CudaInternalDevices &CudaInternalDevices::singleton() {
 
 }  // namespace
 
-unsigned long *CudaInternal::constantMemHostStaging = nullptr;
-cudaEvent_t CudaInternal::constantMemReusable       = nullptr;
-std::mutex CudaInternal::constantMemMutex;
-
 //----------------------------------------------------------------------------
 
+int Impl::CudaInternal::concurrency() {
+  static int const concurrency = m_deviceProp.maxThreadsPerMultiProcessor *
+                                 m_deviceProp.multiProcessorCount;
+  return concurrency;
+}
+
 void CudaInternal::print_configuration(std::ostream &s) const {
   const CudaInternalDevices &dev_info = CudaInternalDevices::singleton();
 
@@ -328,22 +304,13 @@ CudaInternal::~CudaInternal() {
               << std::endl;
   }
 
-  m_cudaDev                 = -1;
-  m_cudaArch                = -1;
-  m_multiProcCount          = 0;
-  m_maxWarpCount            = 0;
-  m_maxBlock                = {0, 0, 0};
-  m_maxSharedWords          = 0;
-  m_maxConcurrency          = 0;
-  m_scratchSpaceCount       = 0;
-  m_scratchFlagsCount       = 0;
-  m_scratchUnifiedCount     = 0;
-  m_scratchUnifiedSupported = 0;
-  m_streamCount             = 0;
-  m_scratchSpace            = nullptr;
-  m_scratchFlags            = nullptr;
-  m_scratchUnified          = nullptr;
-  m_stream                  = nullptr;
+  m_scratchSpaceCount   = 0;
+  m_scratchFlagsCount   = 0;
+  m_scratchUnifiedCount = 0;
+  m_scratchSpace        = nullptr;
+  m_scratchFlags        = nullptr;
+  m_scratchUnified      = nullptr;
+  m_stream              = nullptr;
   for (int i = 0; i < m_n_team_scratch; ++i) {
     m_team_scratch_current_size[i] = 0;
     m_team_scratch_ptr[i]          = nullptr;
@@ -370,15 +337,12 @@ void CudaInternal::fence() const {
   fence("Kokkos::CudaInternal::fence(): Unnamed Instance Fence");
 }
 
-void CudaInternal::initialize(int cuda_device_id, cudaStream_t stream,
-                              bool manage_stream) {
+void CudaInternal::initialize(cudaStream_t stream, bool manage_stream) {
   if (was_finalized)
     Kokkos::abort("Calling Cuda::initialize after Cuda::finalize is illegal\n");
   was_initialized = true;
   if (is_initialized()) return;
 
-  enum { WordSize = sizeof(size_type) };
-
 #ifndef KOKKOS_IMPL_TURN_OFF_CUDA_HOST_INIT_CHECK
   if (!HostSpace::execution_space::impl_is_initialized()) {
     const std::string msg(
@@ -388,107 +352,9 @@ void CudaInternal::initialize(int cuda_device_id, cudaStream_t stream,
   }
 #endif
 
-  const CudaInternalDevices &dev_info = CudaInternalDevices::singleton();
-
   const bool ok_init = nullptr == m_scratchSpace || nullptr == m_scratchFlags;
 
-  const bool ok_id =
-      0 <= cuda_device_id && cuda_device_id < dev_info.m_cudaDevCount;
-
-  // Need device capability 3.0 or better
-
-  const bool ok_dev =
-      ok_id && (3 <= dev_info.m_cudaProp[cuda_device_id].major &&
-                0 <= dev_info.m_cudaProp[cuda_device_id].minor);
-
-  if (ok_init && ok_dev) {
-    const struct cudaDeviceProp &cudaProp = dev_info.m_cudaProp[cuda_device_id];
-
-    m_cudaDev    = cuda_device_id;
-    m_deviceProp = cudaProp;
-
-    KOKKOS_IMPL_CUDA_SAFE_CALL(cudaSetDevice(m_cudaDev));
-    Kokkos::Impl::cuda_device_synchronize(
-        "Kokkos::CudaInternal::initialize: Fence on space initialization");
-
-    // Query what compute capability architecture a kernel executes:
-    m_cudaArch = cuda_kernel_arch();
-
-    if (m_cudaArch == 0) {
-      std::stringstream ss;
-      ss << "Kokkos::Cuda::initialize ERROR: likely mismatch of architecture\n";
-      std::string msg = ss.str();
-      Kokkos::abort(msg.c_str());
-    }
-
-    int compiled_major = m_cudaArch / 100;
-    int compiled_minor = (m_cudaArch % 100) / 10;
-
-    if (compiled_major != cudaProp.major || compiled_minor > cudaProp.minor) {
-      std::stringstream ss;
-      ss << "Kokkos::Cuda::initialize ERROR: running kernels compiled for "
-            "compute capability "
-         << compiled_major << "." << compiled_minor
-         << " on device with compute capability " << cudaProp.major << "."
-         << cudaProp.minor << " is not supported by CUDA!\n";
-      std::string msg = ss.str();
-      Kokkos::abort(msg.c_str());
-    }
-    if (Kokkos::show_warnings() && (compiled_major != cudaProp.major ||
-                                    compiled_minor != cudaProp.minor)) {
-      std::cerr << "Kokkos::Cuda::initialize WARNING: running kernels compiled "
-                   "for compute capability "
-                << compiled_major << "." << compiled_minor
-                << " on device with compute capability " << cudaProp.major
-                << "." << cudaProp.minor
-                << " , this will likely reduce potential performance."
-                << std::endl;
-    }
-
-    // number of multiprocessors
-
-    m_multiProcCount = cudaProp.multiProcessorCount;
-
-    //----------------------------------
-    // Maximum number of warps,
-    // at most one warp per thread in a warp for reduction.
-
-    m_maxWarpCount = cudaProp.maxThreadsPerBlock / Impl::CudaTraits::WarpSize;
-
-    if (Impl::CudaTraits::WarpSize < m_maxWarpCount) {
-      m_maxWarpCount = Impl::CudaTraits::WarpSize;
-    }
-
-    m_maxSharedWords = cudaProp.sharedMemPerBlock / WordSize;
-
-    //----------------------------------
-    // Maximum number of blocks:
-
-    m_maxBlock[0] = cudaProp.maxGridSize[0];
-    m_maxBlock[1] = cudaProp.maxGridSize[1];
-    m_maxBlock[2] = cudaProp.maxGridSize[2];
-
-    m_shmemPerSM       = cudaProp.sharedMemPerMultiprocessor;
-    m_maxShmemPerBlock = cudaProp.sharedMemPerBlock;
-    m_regsPerSM        = cudaProp.regsPerMultiprocessor;
-    m_maxBlocksPerSM =
-        m_cudaArch < 500
-            ? 16
-            : (m_cudaArch < 750 ? 32 : (m_cudaArch == 750 ? 16 : 32));
-    m_maxThreadsPerSM    = cudaProp.maxThreadsPerMultiProcessor;
-    m_maxThreadsPerBlock = cudaProp.maxThreadsPerBlock;
-
-    //----------------------------------
-
-    m_scratchUnifiedSupported = cudaProp.unifiedAddressing;
-
-    if (Kokkos::show_warnings() && !m_scratchUnifiedSupported) {
-      std::cerr << "Kokkos::Cuda device " << cudaProp.name << " capability "
-                << cudaProp.major << "." << cudaProp.minor
-                << " does not support unified virtual address space"
-                << std::endl;
-    }
-
+  if (ok_init) {
     //----------------------------------
     // Multiblock reduction uses scratch flags for counters
     // and scratch space for partial reduction values.
@@ -502,45 +368,10 @@ void CudaInternal::initialize(int cuda_device_id, cudaStream_t stream,
       (void)scratch_flags(reduce_block_count * 2 * sizeof(size_type));
       (void)scratch_space(reduce_block_count * 16 * sizeof(size_type));
     }
-    //----------------------------------
-    // Concurrent bitset for obtaining unique tokens from within
-    // an executing kernel.
-    {
-      m_maxConcurrency = m_maxThreadsPerSM * cudaProp.multiProcessorCount;
-
-      const int32_t buffer_bound =
-          Kokkos::Impl::concurrent_bitset::buffer_bound(m_maxConcurrency);
-
-      // Allocate and initialize uint32_t[ buffer_bound ]
-
-      using Record =
-          Kokkos::Impl::SharedAllocationRecord<Kokkos::CudaSpace, void>;
-
-      Record *const r =
-          Record::allocate(Kokkos::CudaSpace(), "Kokkos::InternalScratchBitset",
-                           sizeof(uint32_t) * buffer_bound);
-
-      Record::increment(r);
-    }
-    //----------------------------------
-
   } else {
     std::ostringstream msg;
-    msg << "Kokkos::Cuda::initialize(" << cuda_device_id << ") FAILED";
-
-    if (!ok_init) {
-      msg << " : Already initialized";
-    }
-    if (!ok_id) {
-      msg << " : Device identifier out of range "
-          << "[0.." << dev_info.m_cudaDevCount << "]";
-    } else if (!ok_dev) {
-      msg << " : Device ";
-      msg << dev_info.m_cudaProp[cuda_device_id].major;
-      msg << ".";
-      msg << dev_info.m_cudaProp[cuda_device_id].minor;
-      msg << " has insufficient capability, required 3.0 or better";
-    }
+    msg << "Kokkos::Cuda::initialize(" << m_cudaDev
+        << ") FAILED : Already initialized";
     Kokkos::Impl::throw_runtime_exception(msg.str());
   }
 
@@ -569,18 +400,12 @@ Kokkos::Cuda::initialize WARNING: Cuda is allocating into UVMSpace by default
   }
 #endif
 
-#ifdef KOKKOS_ENABLE_PRE_CUDA_10_DEPRECATION_API
-  cudaThreadSetCacheConfig(cudaFuncCachePreferShared);
-#else
-  cudaDeviceSetCacheConfig(cudaFuncCachePreferShared);
-#endif
-
   // Init the array for used for arbitrarily sized atomics
-  if (stream == nullptr) Impl::initialize_host_cuda_lock_arrays();
+  if (this == &singleton()) Impl::initialize_host_cuda_lock_arrays();
 
   // Allocate a staging buffer for constant mem in pinned host memory
   // and an event to avoid overwriting driver for previous kernel launches
-  if (stream == nullptr) {
+  if (this == &singleton()) {
     KOKKOS_IMPL_CUDA_SAFE_CALL(
         cudaMallocHost(reinterpret_cast<void **>(&constantMemHostStaging),
                        CudaTraits::ConstantMemoryUsage));
@@ -596,9 +421,9 @@ Kokkos::Cuda::initialize WARNING: Cuda is allocating into UVMSpace by default
   }
 
   KOKKOS_IMPL_CUDA_SAFE_CALL(
-      cudaMalloc(&m_scratch_locks, sizeof(int32_t) * m_maxConcurrency));
+      cudaMalloc(&m_scratch_locks, sizeof(int32_t) * concurrency()));
   KOKKOS_IMPL_CUDA_SAFE_CALL(
-      cudaMemset(m_scratch_locks, 0, sizeof(int32_t) * m_maxConcurrency));
+      cudaMemset(m_scratch_locks, 0, sizeof(int32_t) * concurrency()));
 }
 
 //----------------------------------------------------------------------------
@@ -698,37 +523,44 @@ Cuda::size_type *CudaInternal::scratch_functor(const std::size_t size) const {
   return m_scratchFunctor;
 }
 
-std::pair<void *, int> CudaInternal::resize_team_scratch_space(
-    std::int64_t bytes, bool force_shrink) {
+int CudaInternal::acquire_team_scratch_space() {
+  int current_team_scratch = 0;
+  int zero                 = 0;
+  while (!m_team_scratch_pool[current_team_scratch].compare_exchange_weak(
+      zero, 1, std::memory_order_release, std::memory_order_relaxed)) {
+    current_team_scratch = (current_team_scratch + 1) % m_n_team_scratch;
+  }
+
+  return current_team_scratch;
+}
+
+void *CudaInternal::resize_team_scratch_space(int scratch_pool_id,
+                                              std::int64_t bytes,
+                                              bool force_shrink) {
   // Multiple ParallelFor/Reduce Teams can call this function at the same time
   // and invalidate the m_team_scratch_ptr. We use a pool to avoid any race
   // condition.
-
-  int current_team_scratch = 0;
-  int zero                 = 0;
-  int one                  = 1;
-  while (!m_team_scratch_pool[current_team_scratch].compare_exchange_weak(
-      zero, one, std::memory_order_release, std::memory_order_relaxed)) {
-    current_team_scratch = (current_team_scratch + 1) % m_n_team_scratch;
-  }
-  if (m_team_scratch_current_size[current_team_scratch] == 0) {
-    m_team_scratch_current_size[current_team_scratch] = bytes;
-    m_team_scratch_ptr[current_team_scratch] =
+  if (m_team_scratch_current_size[scratch_pool_id] == 0) {
+    m_team_scratch_current_size[scratch_pool_id] = bytes;
+    m_team_scratch_ptr[scratch_pool_id] =
         Kokkos::kokkos_malloc<Kokkos::CudaSpace>(
             "Kokkos::CudaSpace::TeamScratchMemory",
-            m_team_scratch_current_size[current_team_scratch]);
+            m_team_scratch_current_size[scratch_pool_id]);
   }
-  if ((bytes > m_team_scratch_current_size[current_team_scratch]) ||
-      ((bytes < m_team_scratch_current_size[current_team_scratch]) &&
+  if ((bytes > m_team_scratch_current_size[scratch_pool_id]) ||
+      ((bytes < m_team_scratch_current_size[scratch_pool_id]) &&
        (force_shrink))) {
-    m_team_scratch_current_size[current_team_scratch] = bytes;
-    m_team_scratch_ptr[current_team_scratch] =
+    m_team_scratch_current_size[scratch_pool_id] = bytes;
+    m_team_scratch_ptr[scratch_pool_id] =
         Kokkos::kokkos_realloc<Kokkos::CudaSpace>(
-            m_team_scratch_ptr[current_team_scratch],
-            m_team_scratch_current_size[current_team_scratch]);
+            m_team_scratch_ptr[scratch_pool_id],
+            m_team_scratch_current_size[scratch_pool_id]);
   }
-  return std::make_pair(m_team_scratch_ptr[current_team_scratch],
-                        current_team_scratch);
+  return m_team_scratch_ptr[scratch_pool_id];
+}
+
+void CudaInternal::release_team_scratch_space(int scratch_pool_id) {
+  m_team_scratch_pool[scratch_pool_id] = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -773,15 +605,9 @@ void CudaInternal::finalize() {
   if (m_manage_stream && m_stream != nullptr)
     KOKKOS_IMPL_CUDA_SAFE_CALL(cudaStreamDestroy(m_stream));
 
-  m_cudaDev             = -1;
-  m_multiProcCount      = 0;
-  m_maxWarpCount        = 0;
-  m_maxBlock            = {0, 0, 0};
-  m_maxSharedWords      = 0;
   m_scratchSpaceCount   = 0;
   m_scratchFlagsCount   = 0;
   m_scratchUnifiedCount = 0;
-  m_streamCount         = 0;
   m_scratchSpace        = nullptr;
   m_scratchFlags        = nullptr;
   m_scratchUnified      = nullptr;
@@ -851,8 +677,12 @@ Cuda::size_type Cuda::detect_device_count() {
   return Impl::CudaInternalDevices::singleton().m_cudaDevCount;
 }
 
+#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_4
 int Cuda::concurrency() {
-  return Impl::CudaInternal::singleton().m_maxConcurrency;
+#else
+int Cuda::concurrency() const {
+#endif
+  return Impl::CudaInternal::concurrency();
 }
 
 int Cuda::impl_is_initialized() {
@@ -860,15 +690,120 @@ int Cuda::impl_is_initialized() {
 }
 
 void Cuda::impl_initialize(InitializationSettings const &settings) {
-  Impl::CudaInternal::singleton().initialize(Impl::get_gpu(settings));
+  const int cuda_device_id = Impl::get_gpu(settings);
+  const auto &dev_info     = Impl::CudaInternalDevices::singleton();
 
-  // In order to support setting an atexit hook for Kokkos::finalize
-  // We need to ensure that the Cuda deep_copy instance is not destroyed
-  // before that atexit hook is getting called.
-  // Thus we create the static instance here, so that it will be deallocated
-  // after the potential atexit call.
-  // This is neccessary since we will access that instance in Kokkos::finalize
-  (void)::Kokkos::Impl::cuda_get_deep_copy_space(true);
+  // Need device capability 3.0 or better
+  const bool ok_dev = 3 <= dev_info.m_cudaProp[cuda_device_id].major &&
+                      0 <= dev_info.m_cudaProp[cuda_device_id].minor;
+  if (ok_dev) {
+    const struct cudaDeviceProp &cudaProp = dev_info.m_cudaProp[cuda_device_id];
+
+    Impl::CudaInternal::m_cudaDev    = cuda_device_id;
+    Impl::CudaInternal::m_deviceProp = cudaProp;
+
+    KOKKOS_IMPL_CUDA_SAFE_CALL(cudaSetDevice(cuda_device_id));
+    Kokkos::Impl::cuda_device_synchronize(
+        "Kokkos::CudaInternal::initialize: Fence on space initialization");
+
+    // Query what compute capability architecture a kernel executes:
+    Impl::CudaInternal::m_cudaArch = Impl::cuda_kernel_arch();
+
+    if (Impl::CudaInternal::m_cudaArch == 0) {
+      std::stringstream ss;
+      ss << "Kokkos::Cuda::initialize ERROR: likely mismatch of architecture\n";
+      std::string msg = ss.str();
+      Kokkos::abort(msg.c_str());
+    }
+
+    int compiled_major = Impl::CudaInternal::m_cudaArch / 100;
+    int compiled_minor = (Impl::CudaInternal::m_cudaArch % 100) / 10;
+
+    if ((compiled_major > cudaProp.major) ||
+        ((compiled_major == cudaProp.major) &&
+         (compiled_minor > cudaProp.minor))) {
+      std::stringstream ss;
+      ss << "Kokkos::Cuda::initialize ERROR: running kernels compiled for "
+            "compute capability "
+         << compiled_major << "." << compiled_minor
+         << " on device with compute capability " << cudaProp.major << "."
+         << cudaProp.minor << " is not supported by CUDA!\n";
+      std::string msg = ss.str();
+      Kokkos::abort(msg.c_str());
+    }
+    if (Kokkos::show_warnings() && (compiled_major != cudaProp.major ||
+                                    compiled_minor != cudaProp.minor)) {
+      std::cerr << "Kokkos::Cuda::initialize WARNING: running kernels compiled "
+                   "for compute capability "
+                << compiled_major << "." << compiled_minor
+                << " on device with compute capability " << cudaProp.major
+                << "." << cudaProp.minor
+                << " , this will likely reduce potential performance."
+                << std::endl;
+    }
+
+    // number of multiprocessors
+    Impl::CudaInternal::m_multiProcCount = cudaProp.multiProcessorCount;
+
+    //----------------------------------
+    // Maximum number of warps,
+    // at most one warp per thread in a warp for reduction.
+    Impl::CudaInternal::m_maxWarpCount =
+        cudaProp.maxThreadsPerBlock / Impl::CudaTraits::WarpSize;
+
+    if (Impl::CudaTraits::WarpSize < Impl::CudaInternal::m_maxWarpCount) {
+      Impl::CudaInternal::m_maxWarpCount = Impl::CudaTraits::WarpSize;
+    }
+
+    constexpr auto WordSize = sizeof(size_type);
+    Impl::CudaInternal::m_maxSharedWords =
+        cudaProp.sharedMemPerBlock / WordSize;
+
+    //----------------------------------
+    // Maximum number of blocks:
+
+    Impl::CudaInternal::m_maxBlock[0] = cudaProp.maxGridSize[0];
+    Impl::CudaInternal::m_maxBlock[1] = cudaProp.maxGridSize[1];
+    Impl::CudaInternal::m_maxBlock[2] = cudaProp.maxGridSize[2];
+
+    Impl::CudaInternal::m_shmemPerSM = cudaProp.sharedMemPerMultiprocessor;
+    Impl::CudaInternal::m_maxShmemPerBlock = cudaProp.sharedMemPerBlock;
+    Impl::CudaInternal::m_maxBlocksPerSM =
+        Impl::CudaInternal::m_cudaArch < 500
+            ? 16
+            : (Impl::CudaInternal::m_cudaArch < 750
+                   ? 32
+                   : (Impl::CudaInternal::m_cudaArch == 750 ? 16 : 32));
+    Impl::CudaInternal::m_maxThreadsPerSM =
+        cudaProp.maxThreadsPerMultiProcessor;
+    Impl::CudaInternal::m_maxThreadsPerBlock = cudaProp.maxThreadsPerBlock;
+
+    //----------------------------------
+
+    Impl::CudaInternal::m_scratchUnifiedSupported = cudaProp.unifiedAddressing;
+
+    if (Kokkos::show_warnings() &&
+        !Impl::CudaInternal::m_scratchUnifiedSupported) {
+      std::cerr << "Kokkos::Cuda device " << cudaProp.name << " capability "
+                << cudaProp.major << "." << cudaProp.minor
+                << " does not support unified virtual address space"
+                << std::endl;
+    }
+  } else {
+    std::ostringstream msg;
+    msg << "Kokkos::Cuda::initialize(" << cuda_device_id << ") FAILED: Device ";
+    msg << dev_info.m_cudaProp[cuda_device_id].major;
+    msg << ".";
+    msg << dev_info.m_cudaProp[cuda_device_id].minor;
+    msg << " has insufficient capability, required 3.0 or better";
+    Kokkos::Impl::throw_runtime_exception(msg.str());
+  }
+
+  cudaStream_t singleton_stream;
+  KOKKOS_IMPL_CUDA_SAFE_CALL(cudaStreamCreate(&singleton_stream));
+
+  auto &cuda_singleton = Impl::CudaInternal::singleton();
+  cuda_singleton.initialize(singleton_stream, /*manage*/ true);
 }
 
 std::vector<unsigned> Cuda::detect_device_arch() {
@@ -914,8 +849,7 @@ Cuda::Cuda(cudaStream_t stream, bool manage_stream)
       }) {
   Impl::CudaInternal::singleton().verify_is_initialized(
       "Cuda instance constructor");
-  m_space_instance->initialize(Impl::CudaInternal::singleton().m_cudaDev,
-                               stream, manage_stream);
+  m_space_instance->initialize(stream, manage_stream);
 }
 
 void Cuda::print_configuration(std::ostream &os, bool /*verbose*/) const {
@@ -937,11 +871,9 @@ void Cuda::print_configuration(std::ostream &os, bool /*verbose*/) const {
 #else
   os << "no\n";
 #endif
+#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_4
   os << "  KOKKOS_ENABLE_CUDA_LDG_INTRINSIC: ";
-#ifdef KOKKOS_ENABLE_CUDA_LDG_INTRINSIC
   os << "yes\n";
-#else
-  os << "no\n";
 #endif
   os << "  KOKKOS_ENABLE_CUDA_RELOCATABLE_DEVICE_CODE: ";
 #ifdef KOKKOS_ENABLE_CUDA_RELOCATABLE_DEVICE_CODE
@@ -993,14 +925,6 @@ int g_cuda_space_factory_initialized =
 
 }  // namespace Impl
 
-#ifdef KOKKOS_ENABLE_CXX14
-namespace Tools {
-namespace Experimental {
-constexpr DeviceType DeviceTypeTraits<Cuda>::id;
-}
-}  // namespace Tools
-#endif
-
 }  // namespace Kokkos
 
 #else
