@@ -1,46 +1,18 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 3.0
-//       Copyright (2020) National Technology & Engineering
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
 //               Solutions of Sandia, LLC (NTESS).
 //
 // Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-//
-// ************************************************************************
 //@HEADER
-*/
 
 #ifndef KOKKO_HIP_PARALLEL_TEAM_HPP
 #define KOKKO_HIP_PARALLEL_TEAM_HPP
@@ -59,13 +31,12 @@ namespace Kokkos {
 namespace Impl {
 
 template <typename... Properties>
-class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
+class TeamPolicyInternal<HIP, Properties...>
     : public PolicyTraits<Properties...> {
  public:
   using execution_policy = TeamPolicyInternal;
 
-  using traits    = PolicyTraits<Properties...>;
-  using BlockType = Kokkos::Experimental::Impl::BlockType;
+  using traits = PolicyTraits<Properties...>;
 
   template <typename ExecSpace, typename... OtherProperties>
   friend class TeamPolicyInternal;
@@ -82,7 +53,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
   bool m_tune_vector_length;
 
  public:
-  using execution_space = Kokkos::Experimental::HIP;
+  using execution_space = HIP;
 
   template <class... OtherProperties>
   TeamPolicyInternal(TeamPolicyInternal<OtherProperties...> const& p) {
@@ -119,7 +90,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     using closure_type =
         Impl::ParallelReduce<FunctorType, TeamPolicy<Properties...>,
                              reducer_type>;
-    return internal_team_size_max<closure_type>(f);
+    return internal_team_size_common<BlockType::Max, closure_type>(f);
   }
 
   template <typename FunctorType, typename ReducerType>
@@ -128,7 +99,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     using closure_type =
         Impl::ParallelReduce<FunctorType, TeamPolicy<Properties...>,
                              ReducerType>;
-    return internal_team_size_max<closure_type>(f);
+    return internal_team_size_common<BlockType::Max, closure_type>(f);
   }
 
   template <typename FunctorType>
@@ -151,7 +122,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     using closure_type =
         Impl::ParallelReduce<FunctorType, TeamPolicy<Properties...>,
                              reducer_type>;
-    return internal_team_size_recommended<closure_type>(f);
+    return internal_team_size_common<BlockType::Preferred, closure_type>(f);
   }
 
   template <typename FunctorType, typename ReducerType>
@@ -160,14 +131,12 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     using closure_type =
         Impl::ParallelReduce<FunctorType, TeamPolicy<Properties...>,
                              ReducerType>;
-    return internal_team_size_recommended<closure_type>(f);
+    return internal_team_size_common<BlockType::Preferred, closure_type>(f);
   }
 
   inline bool impl_auto_vector_length() const { return m_tune_vector_length; }
   inline bool impl_auto_team_size() const { return m_tune_team_size; }
-  static int vector_length_max() {
-    return ::Kokkos::Experimental::Impl::HIPTraits::WarpSize;
-  }
+  static int vector_length_max() { return HIPTraits::WarpSize; }
 
   static int verify_requested_vector_length(int requested_vector_length) {
     int test_vector_length =
@@ -176,7 +145,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     // Allow only power-of-two vector_length
     if (!(is_integral_power_of_two(test_vector_length))) {
       int test_pow2           = 1;
-      int constexpr warp_size = Experimental::Impl::HIPTraits::WarpSize;
+      int constexpr warp_size = HIPTraits::WarpSize;
       while (test_pow2 < warp_size) {
         test_pow2 <<= 1;
         if (test_pow2 > test_vector_length) {
@@ -189,19 +158,28 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
     return test_vector_length;
   }
 
-  static int scratch_size_max(int level) {
-    return (
-        level == 0 ? 1024 * 40 :  // FIXME_HIP arbitrarily setting this to 48kB
-            20 * 1024 * 1024);    // FIXME_HIP arbitrarily setting this to 20MB
+  inline static int scratch_size_max(int level) {
+    // HIP Teams use (team_size + 2)*sizeof(double) shared memory for team
+    // reductions. They also use one int64_t in static shared memory for a
+    // shared ID. Furthermore, they use additional scratch memory in some
+    // reduction scenarios, which depend on the size of the value_type and is
+    // NOT captured here
+    constexpr size_t max_possible_team_size = 1024;
+    constexpr size_t max_reserved_shared_mem_per_team =
+        (max_possible_team_size + 2) * sizeof(double) + sizeof(int64_t);
+    // arbitrarily setting level 1 scratch limit to 20MB, for a
+    // MI250 that would give us about 4.4GB for 2 teams per CU
+    constexpr size_t max_l1_scratch_size = 20 * 1024 * 1024;
+
+    size_t max_shmem = HIP().hip_device_prop().sharedMemPerBlock;
+    return (level == 0 ? max_shmem - max_reserved_shared_mem_per_team
+                       : max_l1_scratch_size);
   }
+
   inline void impl_set_vector_length(size_t size) { m_vector_length = size; }
   inline void impl_set_team_size(size_t size) { m_team_size = size; }
   int impl_vector_length() const { return m_vector_length; }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-  KOKKOS_DEPRECATED int vector_length() const { return impl_vector_length(); }
-#endif
-
   int team_size() const { return m_team_size; }
 
   int league_size() const { return m_league_size; }
@@ -229,7 +207,7 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
         m_vector_length(0),
         m_team_scratch_size{0, 0},
         m_thread_scratch_size{0, 0},
-        m_chunk_size(::Kokkos::Experimental::Impl::HIPTraits::WarpSize),
+        m_chunk_size(HIPTraits::WarpSize),
         m_tune_team_size(false),
         m_tune_vector_length(false) {}
 
@@ -245,20 +223,17 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
                 : (verify_requested_vector_length(1))),
         m_team_scratch_size{0, 0},
         m_thread_scratch_size{0, 0},
-        m_chunk_size(::Kokkos::Experimental::Impl::HIPTraits::WarpSize),
+        m_chunk_size(HIPTraits::WarpSize),
         m_tune_team_size(bool(team_size_request <= 0)),
         m_tune_vector_length(bool(vector_length_request <= 0)) {
     // Make sure league size is permissible
-    if (league_size_ >=
-        static_cast<int>(
-            ::Kokkos::Experimental::Impl::hip_internal_maximum_grid_count()[0]))
+    if (league_size_ >= static_cast<int>(hip_internal_maximum_grid_count()[0]))
       Impl::throw_runtime_exception(
           "Requested too large league_size for TeamPolicy on HIP execution "
           "space.");
 
     // Make sure total block size is permissible
-    if (m_team_size * m_vector_length >
-        ::Kokkos::Experimental::Impl::HIPTraits::MaxThreadsPerBlock) {
+    if (m_team_size * m_vector_length > HIPTraits::MaxThreadsPerBlock) {
       Impl::throw_runtime_exception(
           std::string("Kokkos::TeamPolicy< HIP > the team size is too large. "
                       "Team size x vector length must be smaller than 1024."));
@@ -356,57 +331,19 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
 
  protected:
   template <BlockType BlockSize, class ClosureType, class FunctorType>
-  int internal_team_size_common(const FunctorType& f) const {
-    // FIXME_HIP: this could be unified with the
-    // internal_team_size_common_reduce
-    //            once we can turn c++17 constexpr on by default.
-    //            The problem right now is that we can't turn off the evaluation
-    //            of the Analysis' valuesize / StaticValueSize
-
-    const unsigned shmem_block  = team_scratch_size(0) + 2 * sizeof(double);
-    const unsigned shmem_thread = thread_scratch_size(0) + sizeof(double);
-    const int vector_length     = impl_vector_length();
-
-    const auto functor = [&f, shmem_block, shmem_thread, vector_length](
-                             const hipFuncAttributes& attr, int block_size) {
-      int functor_shmem =
-          ::Kokkos::Impl::FunctorTeamShmemSize<FunctorType>::value(
-              f, block_size / vector_length);
-      return shmem_block + shmem_thread * (block_size / vector_length) +
-             functor_shmem + attr.sharedSizeBytes;
-    };
-    int block_size;
-    // FIXME_HIP - could be if constexpr for c++17
-    if (BlockSize == BlockType::Max) {
-      block_size = ::Kokkos::Experimental::Impl::hip_get_max_team_blocksize<
-          ClosureType, typename traits::launch_bounds>(
-          space().impl_internal_space_instance(), functor);
-    } else {
-      block_size =
-          ::Kokkos::Experimental::Impl::hip_get_preferred_team_blocksize<
-              ClosureType, typename traits::launch_bounds>(
-              space().impl_internal_space_instance(), functor);
-    }
-    if (block_size == 0) {
-      Kokkos::Impl::throw_runtime_exception(
-          std::string("Kokkos::Impl::ParallelFor< HIP > could not find a valid "
-                      "team size."));
-    }
-    return block_size / impl_vector_length();
-  }
-
-  template <BlockType BlockSize, class ClosureType, class FunctorType>
-  int internal_team_size_common_reduce(const FunctorType& f) const {
-    using Interface =
-        typename Impl::DeduceFunctorPatternInterface<ClosureType>::type;
-    using Analysis =
-        Impl::FunctorAnalysis<Interface, typename ClosureType::Policy,
-                              FunctorType>;
-
+  int internal_team_size_common(FunctorType const& f) const {
     const unsigned shmem_block = team_scratch_size(0) + 2 * sizeof(double);
-    const unsigned shmem_thread =
-        thread_scratch_size(0) + sizeof(double) +
-        ((Analysis::StaticValueSize != 0) ? 0 : Analysis::value_size(f));
+    unsigned shmem_thread      = thread_scratch_size(0) + sizeof(double);
+    using Tag = typename PatternTagFromImplSpecialization<ClosureType>::type;
+    if constexpr (std::is_same_v<Tag, ParallelReduceTag>) {
+      using Interface =
+          typename Impl::DeduceFunctorPatternInterface<ClosureType>::type;
+      using Analysis =
+          Impl::FunctorAnalysis<Interface, typename ClosureType::Policy,
+                                FunctorType>;
+      shmem_thread +=
+          ((Analysis::StaticValueSize != 0) ? 0 : Analysis::value_size(f));
+    }
     const int vector_length = impl_vector_length();
 
     const auto functor = [&f, shmem_block, shmem_thread, vector_length](
@@ -418,44 +355,36 @@ class TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>
              functor_shmem + attr.sharedSizeBytes;
     };
     int block_size;
-    // FIXME_HIP - could be if constexpr for c++17
-    if (BlockSize == BlockType::Max) {
-      block_size = ::Kokkos::Experimental::Impl::hip_get_max_team_blocksize<
-          ClosureType, typename traits::launch_bounds>(
+    if constexpr (BlockSize == BlockType::Max) {
+      block_size = hip_get_max_team_blocksize<ClosureType,
+                                              typename traits::launch_bounds>(
           space().impl_internal_space_instance(), functor);
     } else {
       block_size =
-          ::Kokkos::Experimental::Impl::hip_get_preferred_team_blocksize<
-              ClosureType, typename traits::launch_bounds>(
+          hip_get_preferred_team_blocksize<ClosureType,
+                                           typename traits::launch_bounds>(
               space().impl_internal_space_instance(), functor);
     }
 
     if (block_size == 0) {
-      Kokkos::Impl::throw_runtime_exception(
-          std::string("Kokkos::Impl::ParallelReduce< HIP > could not find a "
-                      "valid team size."));
+      Kokkos::Impl::throw_runtime_exception(std::string(
+          "Kokkos::Impl::ParallelFor/Reduce< HIP > could not find a valid "
+          "team size."));
+    }
+    if constexpr (std::is_same_v<Tag, ParallelForTag>) {
+      return block_size / impl_vector_length();
+    } else {
+      // Currently we require Power-of-2 team size for reductions.
+      int p2 = 1;
+      while (p2 <= block_size) p2 *= 2;
+      p2 /= 2;
+      return p2 / impl_vector_length();
     }
-    // Currently we require Power-of-2 team size for reductions.
-    int p2 = 1;
-    while (p2 <= block_size) p2 *= 2;
-    p2 /= 2;
-    return p2 / impl_vector_length();
-  }
-
-  template <class ClosureType, class FunctorType>
-  int internal_team_size_max(const FunctorType& f) const {
-    return internal_team_size_common_reduce<BlockType::Max, ClosureType>(f);
-  }
-
-  template <class ClosureType, class FunctorType>
-  int internal_team_size_recommended(const FunctorType& f) const {
-    return internal_team_size_common_reduce<BlockType::Preferred, ClosureType>(
-        f);
   }
 };
 
-__device__ inline int64_t hip_get_scratch_index(
-    Experimental::HIP::size_type league_size, int32_t* scratch_locks) {
+__device__ inline int64_t hip_get_scratch_index(HIP::size_type league_size,
+                                                int32_t* scratch_locks) {
   int64_t threadid = 0;
   __shared__ int64_t base_thread_id;
   if (threadIdx.x == 0 && threadIdx.y == 0) {
@@ -491,12 +420,11 @@ __device__ inline void hip_release_scratch_index(int32_t* scratch_locks,
 }
 
 template <typename FunctorType, typename... Properties>
-class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
-                  Kokkos::Experimental::HIP> {
+class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>, HIP> {
  public:
-  using Policy = TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>;
+  using Policy       = TeamPolicyInternal<HIP, Properties...>;
   using functor_type = FunctorType;
-  using size_type    = ::Kokkos::Experimental::HIP::size_type;
+  using size_type    = HIP::size_type;
 
  private:
   using member_type   = typename Policy::member_type;
@@ -518,10 +446,8 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
   int m_shmem_size;
   void* m_scratch_ptr[2];
   size_t m_scratch_size[2];
+  int m_scratch_pool_id = -1;
   int32_t* m_scratch_locks;
-  // Only let one ParallelFor/Reduce modify the team scratch memory. The
-  // constructor acquires the mutex which is released in the destructor.
-  std::lock_guard<std::mutex> m_scratch_lock_guard;
 
   template <typename TagType>
   __device__ inline std::enable_if_t<std::is_void<TagType>::value> exec_team(
@@ -547,8 +473,7 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
     for (int league_rank = blockIdx.x; league_rank < int_league_size;
          league_rank += gridDim.x) {
       this->template exec_team<work_tag>(typename Policy::member_type(
-          ::Kokkos::Experimental::kokkos_impl_hip_shared_memory<void>(),
-          m_shmem_begin, m_shmem_size,
+          kokkos_impl_hip_shared_memory<void>(), m_shmem_begin, m_shmem_size,
           static_cast<void*>(static_cast<char*>(m_scratch_ptr[1]) +
                              ptrdiff_t(threadid / (blockDim.x * blockDim.y)) *
                                  m_scratch_size[1]),
@@ -566,10 +491,8 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
                      static_cast<int>(m_team_size), 1);
 
     using closure_type =
-        ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
-                    Kokkos::Experimental::HIP>;
-    ::Kokkos::Experimental::Impl::hip_parallel_launch<closure_type,
-                                                      launch_bounds>(
+        ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>, HIP>;
+    Impl::hip_parallel_launch<closure_type, launch_bounds>(
         *this, grid, block, shmem_size_total,
         m_policy.space().impl_internal_space_instance(),
         true);  // copy to device and execute
@@ -580,10 +503,9 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
         m_policy(arg_policy),
         m_league_size(arg_policy.league_size()),
         m_team_size(arg_policy.team_size()),
-        m_vector_size(arg_policy.impl_vector_length()),
-        m_scratch_lock_guard(m_policy.space()
-                                 .impl_internal_space_instance()
-                                 ->m_team_scratch_mutex) {
+        m_vector_size(arg_policy.impl_vector_length()) {
+    auto internal_space_instance =
+        m_policy.space().impl_internal_space_instance();
     m_team_size = m_team_size >= 0 ? m_team_size
                                    : arg_policy.team_size_recommended(
                                          arg_functor, ParallelForTag());
@@ -594,27 +516,26 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
          FunctorTeamShmemSize<FunctorType>::value(m_functor, m_team_size));
     m_scratch_size[0] = m_policy.scratch_size(0, m_team_size);
     m_scratch_size[1] = m_policy.scratch_size(1, m_team_size);
-    m_scratch_locks =
-        m_policy.space().impl_internal_space_instance()->m_scratch_locks;
+    m_scratch_locks   = internal_space_instance->m_scratch_locks;
 
     // Functor's reduce memory, team scan memory, and team shared memory depend
     // upon team size.
     m_scratch_ptr[0] = nullptr;
-    m_scratch_ptr[1] =
-        m_team_size <= 0
-            ? nullptr
-            : m_policy.space()
-                  .impl_internal_space_instance()
-                  ->resize_team_scratch_space(
-                      static_cast<std::int64_t>(m_scratch_size[1]) *
-                      (std::min(static_cast<std::int64_t>(
-                                    Kokkos::Experimental::HIP::concurrency() /
-                                    (m_team_size * m_vector_size)),
-                                static_cast<std::int64_t>(m_league_size))));
+    if (m_team_size <= 0) {
+      m_scratch_ptr[1] = nullptr;
+    } else {
+      m_scratch_pool_id = internal_space_instance->acquire_team_scratch_space();
+      m_scratch_ptr[1]  = internal_space_instance->resize_team_scratch_space(
+          m_scratch_pool_id,
+          static_cast<std::int64_t>(m_scratch_size[1]) *
+              (std::min(
+                  static_cast<std::int64_t>(HIP().concurrency() /
+                                            (m_team_size * m_vector_size)),
+                  static_cast<std::int64_t>(m_league_size))));
+    }
 
     int const shmem_size_total = m_shmem_begin + m_shmem_size;
-    if (m_policy.space().impl_internal_space_instance()->m_maxShmemPerBlock <
-        shmem_size_total) {
+    if (internal_space_instance->m_maxShmemPerBlock < shmem_size_total) {
       Kokkos::Impl::throw_runtime_exception(std::string(
           "Kokkos::Impl::ParallelFor< HIP > insufficient shared memory"));
     }
@@ -625,6 +546,14 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
           "Kokkos::Impl::ParallelFor< HIP > requested too large team size."));
     }
   }
+
+  ~ParallelFor() {
+    if (m_scratch_pool_id >= 0) {
+      m_policy.space()
+          .impl_internal_space_instance()
+          ->release_team_scratch_space(m_scratch_pool_id);
+    }
+  }
 };
 
 //----------------------------------------------------------------------------
@@ -632,9 +561,9 @@ class ParallelFor<FunctorType, Kokkos::TeamPolicy<Properties...>,
 
 template <class FunctorType, class ReducerType, class... Properties>
 class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
-                     ReducerType, Kokkos::Experimental::HIP> {
+                     ReducerType, HIP> {
  public:
-  using Policy = TeamPolicyInternal<Kokkos::Experimental::HIP, Properties...>;
+  using Policy = TeamPolicyInternal<HIP, Properties...>;
 
  private:
   using member_type   = typename Policy::member_type;
@@ -658,7 +587,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
 
  public:
   using functor_type = FunctorType;
-  using size_type    = Kokkos::Experimental::HIP::size_type;
+  using size_type    = HIP::size_type;
 
   static int constexpr UseShflReduction = (analysis::StaticValueSize != 0);
 
@@ -687,13 +616,11 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
   size_type m_shmem_size;
   void* m_scratch_ptr[2];
   size_t m_scratch_size[2];
+  int m_scratch_pool_id = -1;
   int32_t* m_scratch_locks;
   const size_type m_league_size;
   int m_team_size;
   const size_type m_vector_size;
-  // Only let one ParallelFor/Reduce modify the team scratch memory. The
-  // constructor acquires the mutex which is released in the destructor.
-  std::lock_guard<std::mutex> m_scratch_lock_guard;
 
   template <class TagType>
   __device__ inline std::enable_if_t<std::is_void<TagType>::value> exec_team(
@@ -714,8 +641,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
          league_rank += gridDim.x) {
       this->template exec_team<work_tag>(
           member_type(
-              Kokkos::Experimental::kokkos_impl_hip_shared_memory<char>() +
-                  m_team_begin,
+              kokkos_impl_hip_shared_memory<char>() + m_team_begin,
               m_shmem_begin, m_shmem_size,
               reinterpret_cast<void*>(
                   reinterpret_cast<char*>(m_scratch_ptr[1]) +
@@ -752,9 +678,9 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
                        reducer_conditional::select(m_functor, m_reducer)) /
                    sizeof(size_type));
 
-    reference_type value = final_reducer.init(
-        Kokkos::Experimental::kokkos_impl_hip_shared_memory<size_type>() +
-        threadIdx.y * word_count.value);
+    reference_type value =
+        final_reducer.init(kokkos_impl_hip_shared_memory<size_type>() +
+                           threadIdx.y * word_count.value);
 
     // Iterate this block through the league
     iterate_through_league(threadid, value);
@@ -765,16 +691,14 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
       do_final_reduce =
           hip_single_inter_block_reduce_scan<false, FunctorType, work_tag>(
               reducer_conditional::select(m_functor, m_reducer), blockIdx.x,
-              gridDim.x,
-              Kokkos::Experimental::kokkos_impl_hip_shared_memory<size_type>(),
+              gridDim.x, kokkos_impl_hip_shared_memory<size_type>(),
               m_scratch_space, m_scratch_flags);
     if (do_final_reduce) {
       // This is the final block with the final result at the final threads'
       // location
 
-      size_type* const shared =
-          Kokkos::Experimental::kokkos_impl_hip_shared_memory<size_type>() +
-          (blockDim.y - 1) * word_count.value;
+      size_type* const shared = kokkos_impl_hip_shared_memory<size_type>() +
+                                (blockDim.y - 1) * word_count.value;
       size_type* const global = m_result_ptr_device_accessible
                                     ? reinterpret_cast<size_type*>(m_result_ptr)
                                     : m_scratch_space;
@@ -783,7 +707,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         final_reducer.final(reinterpret_cast<value_type*>(shared));
       }
 
-      if (Kokkos::Experimental::Impl::HIPTraits::WarpSize < word_count.value) {
+      if (HIPTraits::WarpSize < word_count.value) {
         __syncthreads();
       }
 
@@ -836,18 +760,15 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
     if (!is_empty_range || need_device_set) {
       const int block_count =
           UseShflReduction
-              ? std::min(
-                    m_league_size,
-                    size_type(1024 *
-                              Kokkos::Experimental::Impl::HIPTraits::WarpSize))
+              ? std::min(m_league_size, size_type(1024 * HIPTraits::WarpSize))
               : std::min(static_cast<int>(m_league_size), m_team_size);
 
-      m_scratch_space = Kokkos::Experimental::Impl::hip_internal_scratch_space(
+      m_scratch_space = hip_internal_scratch_space(
           m_policy.space(), analysis::value_size(reducer_conditional::select(
                                 m_functor, m_reducer)) *
                                 block_count);
-      m_scratch_flags = Kokkos::Experimental::Impl::hip_internal_scratch_flags(
-          m_policy.space(), sizeof(size_type));
+      m_scratch_flags =
+          hip_internal_scratch_flags(m_policy.space(), sizeof(size_type));
 
       dim3 block(m_vector_size, m_team_size, 1);
       dim3 grid(block_count, 1, 1);
@@ -859,9 +780,8 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
 
       using closure_type =
           ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
-                         ReducerType, Kokkos::Experimental::HIP>;
-      Kokkos::Experimental::Impl::hip_parallel_launch<closure_type,
-                                                      launch_bounds>(
+                         ReducerType, HIP>;
+      Impl::hip_parallel_launch<closure_type, launch_bounds>(
           *this, grid, block, shmem_size_total,
           m_policy.space().impl_internal_space_instance(),
           true);  // copy to device and execute
@@ -872,8 +792,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         if (m_result_ptr) {
           const int size = analysis::value_size(
               reducer_conditional::select(m_functor, m_reducer));
-          DeepCopy<HostSpace, Kokkos::Experimental::HIPSpace>(
-              m_result_ptr, m_scratch_space, size);
+          DeepCopy<HostSpace, HIPSpace>(m_result_ptr, m_scratch_space, size);
         }
       }
     } else {
@@ -893,7 +812,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         m_reducer(InvalidType()),
         m_result_ptr(arg_result.data()),
         m_result_ptr_device_accessible(
-            MemorySpaceAccess<Kokkos::Experimental::HIPSpace,
+            MemorySpaceAccess<HIPSpace,
                               typename ViewType::memory_space>::accessible),
         m_result_ptr_host_accessible(
             MemorySpaceAccess<Kokkos::HostSpace,
@@ -906,10 +825,9 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         m_scratch_ptr{nullptr, nullptr},
         m_league_size(arg_policy.league_size()),
         m_team_size(arg_policy.team_size()),
-        m_vector_size(arg_policy.impl_vector_length()),
-        m_scratch_lock_guard(m_policy.space()
-                                 .impl_internal_space_instance()
-                                 ->m_team_scratch_mutex) {
+        m_vector_size(arg_policy.impl_vector_length()) {
+    auto internal_space_instance =
+        m_policy.space().impl_internal_space_instance();
     m_team_size = m_team_size >= 0 ? m_team_size
                                    : arg_policy.team_size_recommended(
                                          arg_functor, ParallelReduceTag());
@@ -926,19 +844,19 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         FunctorTeamShmemSize<FunctorType>::value(arg_functor, m_team_size);
     m_scratch_size[0] = m_shmem_size;
     m_scratch_size[1] = m_policy.scratch_size(1, m_team_size);
-    m_scratch_locks =
-        m_policy.space().impl_internal_space_instance()->m_scratch_locks;
-    m_scratch_ptr[1] =
-        m_team_size <= 0
-            ? nullptr
-            : m_policy.space()
-                  .impl_internal_space_instance()
-                  ->resize_team_scratch_space(
-                      static_cast<std::int64_t>(m_scratch_size[1]) *
-                      (std::min(static_cast<std::int64_t>(
-                                    Kokkos::Experimental::HIP::concurrency() /
-                                    (m_team_size * m_vector_size)),
-                                static_cast<std::int64_t>(m_league_size))));
+    m_scratch_locks   = internal_space_instance->m_scratch_locks;
+    if (m_team_size <= 0) {
+      m_scratch_ptr[1] = nullptr;
+    } else {
+      m_scratch_pool_id = internal_space_instance->acquire_team_scratch_space();
+      m_scratch_ptr[1]  = internal_space_instance->resize_team_scratch_space(
+          m_scratch_pool_id,
+          static_cast<std::int64_t>(m_scratch_size[1]) *
+              (std::min(
+                  static_cast<std::int64_t>(HIP().concurrency() /
+                                            (m_team_size * m_vector_size)),
+                  static_cast<std::int64_t>(m_league_size))));
+    }
 
     // The global parallel_reduce does not support vector_length other than 1 at
     // the moment
@@ -948,8 +866,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
           "greater than 1 is not currently supported for HIP for dynamic "
           "sized reduction types.");
 
-    if ((m_team_size < Kokkos::Experimental::Impl::HIPTraits::WarpSize) &&
-        !UseShflReduction)
+    if ((m_team_size < HIPTraits::WarpSize) && !UseShflReduction)
       Impl::throw_runtime_exception(
           "Kokkos::parallel_reduce with a TeamPolicy using a team_size smaller "
           "than 64 is not currently supported with HIP for dynamic sized "
@@ -966,8 +883,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
           std::string("Kokkos::Impl::ParallelReduce< HIP > bad team size"));
     }
 
-    if (m_policy.space().impl_internal_space_instance()->m_maxShmemPerBlock <
-        shmem_size_total) {
+    if (internal_space_instance->m_maxShmemPerBlock < shmem_size_total) {
       Kokkos::Impl::throw_runtime_exception(
           std::string("Kokkos::Impl::ParallelReduce< HIP > requested too much "
                       "L0 scratch memory"));
@@ -989,9 +905,8 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         m_reducer(reducer),
         m_result_ptr(reducer.view().data()),
         m_result_ptr_device_accessible(
-            MemorySpaceAccess<Kokkos::Experimental::HIPSpace,
-                              typename ReducerType::result_view_type::
-                                  memory_space>::accessible),
+            MemorySpaceAccess<HIPSpace, typename ReducerType::result_view_type::
+                                            memory_space>::accessible),
         m_result_ptr_host_accessible(
             MemorySpaceAccess<Kokkos::HostSpace,
                               typename ReducerType::result_view_type::
@@ -1004,10 +919,9 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         m_scratch_ptr{nullptr, nullptr},
         m_league_size(arg_policy.league_size()),
         m_team_size(arg_policy.team_size()),
-        m_vector_size(arg_policy.impl_vector_length()),
-        m_scratch_lock_guard(m_policy.space()
-                                 .impl_internal_space_instance()
-                                 ->m_team_scratch_mutex) {
+        m_vector_size(arg_policy.impl_vector_length()) {
+    auto internal_space_instance =
+        m_policy.space().impl_internal_space_instance();
     m_team_size = m_team_size >= 0
                       ? m_team_size
                       : arg_policy.team_size_recommended(arg_functor, reducer,
@@ -1024,19 +938,19 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
         FunctorTeamShmemSize<FunctorType>::value(arg_functor, m_team_size);
     m_scratch_size[0] = m_shmem_size;
     m_scratch_size[1] = m_policy.scratch_size(1, m_team_size);
-    m_scratch_locks =
-        m_policy.space().impl_internal_space_instance()->m_scratch_locks;
-    m_scratch_ptr[1] =
-        m_team_size <= 0
-            ? nullptr
-            : m_policy.space()
-                  .impl_internal_space_instance()
-                  ->resize_team_scratch_space(
-                      static_cast<std::int64_t>(m_scratch_size[1]) *
-                      (std::min(static_cast<std::int64_t>(
-                                    Kokkos::Experimental::HIP::concurrency() /
-                                    (m_team_size * m_vector_size)),
-                                static_cast<std::int64_t>(m_league_size))));
+    m_scratch_locks   = internal_space_instance->m_scratch_locks;
+    if (m_team_size <= 0) {
+      m_scratch_ptr[1] = nullptr;
+    } else {
+      m_scratch_pool_id = internal_space_instance->acquire_team_scratch_space();
+      m_scratch_ptr[1]  = internal_space_instance->resize_team_scratch_space(
+          m_scratch_pool_id,
+          static_cast<std::int64_t>(m_scratch_size[1]) *
+              (std::min(
+                  static_cast<std::int64_t>(HIP().concurrency() /
+                                            (m_team_size * m_vector_size)),
+                  static_cast<std::int64_t>(m_league_size))));
+    }
 
     // The global parallel_reduce does not support vector_length other than 1 at
     // the moment
@@ -1046,8 +960,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
           "greater than 1 is not currently supported for HIP for dynamic "
           "sized reduction types.");
 
-    if ((m_team_size < Kokkos::Experimental::Impl::HIPTraits::WarpSize) &&
-        !UseShflReduction)
+    if ((m_team_size < HIPTraits::WarpSize) && !UseShflReduction)
       Impl::throw_runtime_exception(
           "Kokkos::parallel_reduce with a TeamPolicy using a team_size smaller "
           "than 64 is not currently supported with HIP for dynamic sized "
@@ -1059,8 +972,7 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
     const int shmem_size_total = m_team_begin + m_shmem_begin + m_shmem_size;
     if ((!Kokkos::Impl::is_integral_power_of_two(m_team_size) &&
          !UseShflReduction) ||
-        m_policy.space().impl_internal_space_instance()->m_maxShmemPerBlock <
-            shmem_size_total) {
+        internal_space_instance->m_maxShmemPerBlock < shmem_size_total) {
       Kokkos::Impl::throw_runtime_exception(
           std::string("Kokkos::Impl::ParallelReduce< HIP > bad team size"));
     }
@@ -1073,6 +985,14 @@ class ParallelReduce<FunctorType, Kokkos::TeamPolicy<Properties...>,
                       "large team size."));
     }
   }
+
+  ~ParallelReduce() {
+    if (m_scratch_pool_id >= 0) {
+      m_policy.space()
+          .impl_internal_space_instance()
+          ->release_team_scratch_space(m_scratch_pool_id);
+    }
+  }
 };
 }  // namespace Impl
 }  // namespace Kokkos
