@@ -1,58 +1,26 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 3.0
-//       Copyright (2020) National Technology & Engineering
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
 //               Solutions of Sandia, LLC (NTESS).
 //
 // Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-//
-// ************************************************************************
 //@HEADER
-*/
 
 /// \file Kokkos_Parallel.hpp
 /// \brief Declaration of parallel operators
 
 #ifndef KOKKOS_IMPL_PUBLIC_INCLUDE
 #include <Kokkos_Macros.hpp>
-#ifndef KOKKOS_ENABLE_DEPRECATED_CODE_3
 static_assert(false,
               "Including non-public Kokkos header files is not allowed.");
-#else
-KOKKOS_IMPL_WARNING("Including non-public Kokkos header files is not allowed.")
-#endif
 #endif
 #ifndef KOKKOS_PARALLEL_HPP
 #define KOKKOS_PARALLEL_HPP
@@ -185,18 +153,6 @@ inline void parallel_for(
   Kokkos::parallel_for("", policy, functor);
 }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-template <class ExecPolicy, class FunctorType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_for(
-    const ExecPolicy& policy, const FunctorType& functor,
-    const std::string& str,
-    std::enable_if_t<is_execution_policy<ExecPolicy>::value>* = nullptr) {
-  Kokkos::parallel_for(str, policy, functor);
-}
-#endif
-
 template <class FunctorType>
 inline void parallel_for(const std::string& str, const size_t work_count,
                          const FunctorType& functor) {
@@ -214,16 +170,6 @@ inline void parallel_for(const size_t work_count, const FunctorType& functor) {
   ::Kokkos::parallel_for("", work_count, functor);
 }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-template <class FunctorType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_for(const size_t work_count, const FunctorType& functor,
-                         const std::string& str) {
-  ::Kokkos::parallel_for(str, work_count, functor);
-}
-#endif
-
 }  // namespace Kokkos
 
 #include <Kokkos_Parallel_Reduce.hpp>
@@ -423,18 +369,6 @@ inline void parallel_scan(
   ::Kokkos::parallel_scan("", policy, functor);
 }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-template <class ExecutionPolicy, class FunctorType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_scan(
-    const ExecutionPolicy& policy, const FunctorType& functor,
-    const std::string& str,
-    std::enable_if_t<is_execution_policy<ExecutionPolicy>::value>* = nullptr) {
-  ::Kokkos::parallel_scan(str, policy, functor);
-}
-#endif
-
 template <class FunctorType>
 inline void parallel_scan(const std::string& str, const size_t work_count,
                           const FunctorType& functor) {
@@ -453,16 +387,6 @@ inline void parallel_scan(const size_t work_count, const FunctorType& functor) {
   ::Kokkos::parallel_scan("", work_count, functor);
 }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-template <class FunctorType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_scan(const size_t work_count, const FunctorType& functor,
-                          const std::string& str) {
-  ::Kokkos::parallel_scan(str, work_count, functor);
-}
-#endif
-
 template <class ExecutionPolicy, class FunctorType, class ReturnType,
           class Enable =
               std::enable_if_t<is_execution_policy<ExecutionPolicy>::value>>
@@ -473,17 +397,26 @@ inline void parallel_scan(const std::string& str, const ExecutionPolicy& policy,
   ExecutionPolicy inner_policy = policy;
   Kokkos::Tools::Impl::begin_parallel_scan(inner_policy, functor, str, kpID);
 
-  Kokkos::Impl::shared_allocation_tracking_disable();
-  Impl::ParallelScanWithTotal<FunctorType, ExecutionPolicy, ReturnType> closure(
-      functor, inner_policy, return_value);
-  Kokkos::Impl::shared_allocation_tracking_enable();
-
-  closure.execute();
+  if constexpr (Kokkos::is_view<ReturnType>::value) {
+    Kokkos::Impl::shared_allocation_tracking_disable();
+    Impl::ParallelScanWithTotal<FunctorType, ExecutionPolicy, ReturnType>
+        closure(functor, inner_policy, return_value);
+    Kokkos::Impl::shared_allocation_tracking_enable();
+    closure.execute();
+  } else {
+    Kokkos::Impl::shared_allocation_tracking_disable();
+    Kokkos::View<ReturnType, Kokkos::HostSpace> view(&return_value);
+    Impl::ParallelScanWithTotal<FunctorType, ExecutionPolicy, ReturnType>
+        closure(functor, inner_policy, view);
+    Kokkos::Impl::shared_allocation_tracking_enable();
+    closure.execute();
+  }
 
   Kokkos::Tools::Impl::end_parallel_scan(inner_policy, functor, str, kpID);
 
-  policy.space().fence(
-      "Kokkos::parallel_scan: fence due to result being a value, not a view");
+  if (!Kokkos::is_view<ReturnType>::value)
+    policy.space().fence(
+        "Kokkos::parallel_scan: fence due to result being a value, not a view");
 }
 
 template <class ExecutionPolicy, class FunctorType, class ReturnType>
@@ -494,18 +427,6 @@ inline void parallel_scan(
   ::Kokkos::parallel_scan("", policy, functor, return_value);
 }
 
-#ifdef KOKKOS_ENABLE_DISABLE_DEPRECATED_CODE_3
-template <class ExecutionPolicy, class FunctorType, class ReturnType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_scan(
-    const ExecutionPolicy& policy, const FunctorType& functor,
-    ReturnType& return_value, const std::string& str,
-    std::enable_if_t<is_execution_policy<ExecutionPolicy>::value>* = nullptr) {
-  ::Kokkos::parallel_scan(str, policy, functor, return_value);
-}
-#endif
-
 template <class FunctorType, class ReturnType>
 inline void parallel_scan(const std::string& str, const size_t work_count,
                           const FunctorType& functor,
@@ -526,16 +447,6 @@ inline void parallel_scan(const size_t work_count, const FunctorType& functor,
   ::Kokkos::parallel_scan("", work_count, functor, return_value);
 }
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_3
-template <class FunctorType, class ReturnType>
-KOKKOS_DEPRECATED_WITH_COMMENT(
-    "Use the overload taking the label as first argument instead!")
-inline void parallel_scan(const size_t work_count, const FunctorType& functor,
-                          ReturnType& return_value, const std::string& str) {
-  ::Kokkos::parallel_scan(str, work_count, functor, return_value);
-}
-#endif
-
 }  // namespace Kokkos
 
 //----------------------------------------------------------------------------
