@@ -1,46 +1,18 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 3.0
-//       Copyright (2020) National Technology & Engineering
+//                        Kokkos v. 4.0
+//       Copyright (2022) National Technology & Engineering
 //               Solutions of Sandia, LLC (NTESS).
 //
 // Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
+// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
+// See https://kokkos.org/LICENSE for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// 1. Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright
-// notice, this list of conditions and the following disclaimer in the
-// documentation and/or other materials provided with the distribution.
-//
-// 3. Neither the name of the Corporation nor the names of the
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
-// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
-// Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-//
-// ************************************************************************
 //@HEADER
-*/
 
 #ifndef KOKKOS_COMBINED_REDUCER_HPP
 #define KOKKOS_COMBINED_REDUCER_HPP
@@ -51,7 +23,6 @@
 #include <Kokkos_Parallel_Reduce.hpp>
 #include <Kokkos_ExecPolicy.hpp>
 #include <Kokkos_AnonymousSpace.hpp>
-#include <impl/Kokkos_Utilities.hpp>  // comma operator fold emulation
 
 #include <utility>
 
@@ -152,15 +123,11 @@ struct CombinedReducerStorageImpl {
   // model Reducer
 
   KOKKOS_INLINE_FUNCTION
-  constexpr _fold_comma_emulation_return _init(value_type& val) const {
-    m_reducer.init(val);
-    return _fold_comma_emulation_return{};
-  }
+  constexpr void _init(value_type& val) const { m_reducer.init(val); }
 
-  KOKKOS_INLINE_FUNCTION constexpr _fold_comma_emulation_return _join(
-      value_type& dest, value_type const& src) const {
+  KOKKOS_INLINE_FUNCTION constexpr void _join(value_type& dest,
+                                              value_type const& src) const {
     m_reducer.join(dest, src);
-    return _fold_comma_emulation_return{};
   }
 };
 
@@ -231,16 +198,16 @@ struct CombinedReducerImpl<std::integer_sequence<size_t, Idxs...>, Space,
 
   KOKKOS_FUNCTION constexpr void join(value_type& dest,
                                       value_type const& src) const noexcept {
-    emulate_fold_comma_operator(
-        this->CombinedReducerStorageImpl<Idxs, Reducers>::_join(
-            dest.template get<Idxs, typename Reducers::value_type>(),
-            src.template get<Idxs, typename Reducers::value_type>())...);
+    (this->CombinedReducerStorageImpl<Idxs, Reducers>::_join(
+         dest.template get<Idxs, typename Reducers::value_type>(),
+         src.template get<Idxs, typename Reducers::value_type>()),
+     ...);
   }
 
   KOKKOS_FUNCTION constexpr void init(value_type& dest) const noexcept {
-    emulate_fold_comma_operator(
-        this->CombinedReducerStorageImpl<Idxs, Reducers>::_init(
-            dest.template get<Idxs, typename Reducers::value_type>())...);
+    (this->CombinedReducerStorageImpl<Idxs, Reducers>::_init(
+         dest.template get<Idxs, typename Reducers::value_type>()),
+     ...);
   }
 
   // TODO figure out if we also need to call through to final
@@ -274,11 +241,11 @@ struct CombinedReducerImpl<std::integer_sequence<size_t, Idxs...>, Space,
   static void write_value_back_to_original_references(
       const ExecutionSpace& exec_space, value_type const& value,
       Reducers const&... reducers_that_reference_original_values) noexcept {
-    emulate_fold_comma_operator(
-        (write_one_value_back<ExecutionSpace, Idxs>(
-             exec_space, reducers_that_reference_original_values.view(),
-             value.template get<Idxs, typename Reducers::value_type>()),
-         0)...);
+    (write_one_value_back<ExecutionSpace, Idxs>(
+         exec_space, reducers_that_reference_original_values.view(),
+         value.template get<Idxs, typename Reducers::value_type>()),
+
+     ...);
   }
 };
 
